@@ -1,4 +1,4 @@
-## conversion.R (2017-01-19)
+## conversion.R (2017-12-17)
 
 ##   Conversion Among Allelic Data Classes
 
@@ -7,11 +7,13 @@
 ## This file is part of the R-package `pegas'.
 ## See the file ../DESCRIPTION for licensing issues.
 
-loci2genind <- function(x, ploidy = 2, na.alleles = c("0", "."))
+loci2genind <- function(x, ploidy = 2, na.alleles = c("0", "."), unphase = TRUE)
 {
     ipop <- which(names(x) == "population")
     pop <- if (length(ipop)) x[, ipop] else NULL
 
+    if (unphase) x <- unphase(x)
+
     if (any(isDot <- na.alleles == ".")) na.alleles[isDot] <- "\\."
     pat <- c(paste0("^", na.alleles, "/"), paste0("/", na.alleles, "$"), paste0("/", na.alleles, "/"))
     pat <- paste(pat, collapse = "|")
@@ -57,7 +59,8 @@ genind2loci <- function(x) as.loci.genind(x)
         Sys.setlocale("LC_COLLATE", "C")
         on.exit(Sys.setlocale("LC_COLLATE", locale))
     }
-    if (index.only) order(x) else sort(x)
+    o <- sort.list(x) # faster than order(x)
+    if (index.only) o else x[o] # faster than sort()
 }
 
 .check.order.alleles <- function(x)
